# -*- coding: utf-8 -*-

meas_dict = {
             "%": "percent",
             "°C": "degree Celsius",
             "°F": "degree Fahrenheit",
             "F": "Fahrenheit",
             "°Re": "degree Reaumur",
             "amu": "atomic mass unit",
             "au": "astronomical unit",
             "BTC": "Bitcoin",
             "bps": "bit per second",
             "Bq": "becquerel",
             "BTU": "British Thermal Unit",
             "B/s": "byte per second",
             "bu": "bushel",
             "cbl": "cable length",
             "cal": "calorie",
             "ct": "carat",
             "cc": "cubic centimetre",
             "ccf": "centum cubic foot",
             "cg": "centigram",
             "cl": "centilitre",
             "cm": "centimetre",
             "cm²": "square centimetre",
             "cu": "cubit",
             "dm³": "cubic decimetre",
             "cm³": "cubic centimetre",
             "ft³": "cubic foot",
             "in³": "cubic inch",
             "km³": "cubic kilometre",
             "m³": "cubic metre",
             "mi³": "cubic mile",
             "mm³": "cubic millimetre",
             "pc³": "cubic parsec",
             "yd³": "cubic yard",
             "Ci": "curie",
             "dN": "decanewton",
             "dl": "decilitre",
             "dm": "decimetre",
             "deg": "degree",
             "dr": "dram",
             "eV": "electronvolt",
             "fath": "fathom",
             "ft": "foot",
             "ft²": "square foot",
             "fur": "furlong",
             "g": "gram",
             "gal": "gallon",
             "Gal": "Galileo",
             "Gb": "gigabyte",
             "GiB": "gigabyte",
             "Gbit": "gigabit",
             "Gibit": "gigabit",
             "GBq": "gigabecquerel",
             "gf": "gravet-force",
             "Gf": "grave-force",
             "gph": "gallon per hour",
             "gpm": "gallon per minute",
             "gps": "gallon per second",
             "Gy": "gray",
             "hd": "hand",
             "ha": "hectare",
             "hl": "hectolitre",
             "hPa": "hectopascal",
             "hp": "horsepower",
             "hZ": "hertz",
             "in.": "inch",
             "in²": "square inch",
             "in³": "cubic inch",
             "J": "joule",
             "K": "kelvin",
             "kHz": "kilohertz",
             "Kb": "kilobyte",
             "KiB": "kilobyte",
             "Kbit": "kilobit",
             "Kibit": "kilobit",
             "kbps": "kilobit per second",
             "kBq": "kilobecquerel",
             "kcal": "kilocalorie",
             "kg": "kilogram",
             "kJ": "kilojoule",
             "kN": "kilonewton",
             "kPa": "kilopascal",
             "kW": "kilowatt",
             "km": "kilometre",
             "km²": "square kilometre",
             "km³": "cubic kilometre",
             "kn": "knot",
             "l": "litre",
             "L": "Planck length",
             "L²": "Planck area",
             "L³": "Planck volume",
             "lb": "pound",
             "m": "metre",
             "M": "Planck mass",
             "m²": "square metre",
             "Mb": "megabyte",
             "MiB": "megabyte",
             "Mbps": "megabyte per second",
             "Mcal": "megacalorie",
             "mcg": "microgram",
             "mCi": "millicurie",
             "MBq": "megabecquerel",
             "Mbit": "megabit",
             "Mibit": "megabit",
             "MJ": "megajoule",
             "MN": "meganewton",
             "Mpa": "megapascal",
             "MW": "megawatt",
             "mkg": "metre-kilogram",
             "mg": "milligram",
             "RT": "metric refrigeration ton",
             "mi": "mile",
             "MHz": "megahertz",
             "GHz": "gigahertz",
             "mi³": "cubic mile",
             "mpg": "mile per gallon",
             "mph": "mile per hour",
             "m.p.h": "mile per hour",
             "ml": "millilitre",
             "mm": "millimetre",
             "mN": "millinewton",
             "min": "minute",
             "mm²": "square millimetre",
             "mol": "mole",
             "N": "newton",
             "nm": "nanometre",
             "Ncm": "newton centimetre",
             "Nm": "newton metre",
             "oz": "ounce",
             "ozf": "ounce force",
             "Pa": "pascal",
             "plm": "palm",
             "pc": "parsec",
             "ppm": "part per million",
             "ppb": "part per billion",
             "Pb": "petabyte",
             "pc³": "cubic parsec",
             "pdl": "poundal",
             "pt": "pint",
             "px": "pixel",
             "qt": "quart",
             "rad": "radian",
             "rd": "rod",
             "s": "second",
             "Sv": "sievert",
             "dm²": "square decimetre",
             "yd²": "square yard",
             "T": "Planck time",
             "Tbit": "terabit",
             "Tb": "terabyte",
             "tsp": "teaspoon",
             "tbsp": "tablespoon",
             "th": "thou",
             "VA": "volt-ampere",
             "V": "volt",
             "W": "watt",
             "yd": "yard",
             }

meas_dict_pl = {
                "%": "percent",
                "°C": "degrees Celsius",
                "°F": "degrees Fahrenheit",
                "F": "Fahrenheit",
                "°Re": "degrees Reaumur",
                "amu": "atomic mass units",
                "atm": "atmospheres",
                "au": "astronomical units",
                "BTC": "Bitcoins",
                "bps": "bits per second",
                "Bq": "becquerels",
                "BTU": "British Thermal Units",
                "B/s": "bytes per second",
                "bu": "bushels",
                "cbl": "cable lengths",
                "cal": "calories",
                "ct": "carats",
                "cc": "cubic centimetres",
                "ccf": "centum cubic feet",
                "cg": "centigrams",
                "cl": "centilitres",
                "cm": "centimetres",
                "cm²": "square centimetres",
                "cu": "cubits",
                "dm³": "cubic decimetres",
                "cm³": "cubic centimetres",
                "ft³": "cubic feet",
                "in³": "cubic inches",
                "km³": "cubic kilometres",
                "m³": "cubic metres",
                "mi³": "cubic miles",
                "mm³": "cubic millimetres",
                "pc³": "cubic parsecs",
                "yd³": "cubic yards",
                "Ci": "curies",
                "dN": "decanewtons",
                "dl": "decilitres",
                "dm": "decimetres",
                "deg": "degrees",
                "dr": "drams",
                "eV": "electronvolts",
                "fath": "fathoms",
                "ft": "feet",
                "ft²": "square feet",
                "fur": "furlongs",
                "g": "grams",
                "gal": "gallons",
                "Gal": "Galileos",
                "Gb": "gigabytes",
                "GiB": "gigabytes",
                "Gbit": "gigabits",
                "Gibit": "gigabits",
                "GBq": "gigabecquerels",
                "gf": "gravet-force",
                "Gf": "grave-force",
                "gph": "gallons per hour",
                "gpm": "gallons per minute",
                "gps": "gallons per second",
                "Gy": "grays",
                "hd": "hands",
                "ha": "hectares",
                "hl": "hectolitres",
                "hPa": "hectopascals",
                "hp": "horsepower",
                "hZ": "hertz",
                "in.": "inches",
                "in²": "square inches",
                "in³": "cubic inches",
                "J": "joules",
                "K": "kelvin",
                "kHz": "kilohertz",
                "Kb": "kilobytes",
                "KiB": "kilobytes",
                "Kbit": "kilobits",
                "Kibit": "kilobits",
                "kbps": "kilobits per second",
                "kBq": "kilobecquerels",
                "kcal": "kilocalories",
                "kg": "kilograms",
                "kJ": "kilojoules",
                "kN": "kilonewtons",
                "kPa": "kilopascals",
                "kW": "kilowatts",
                "km": "kilometres",
                "km²": "square kilometres",
                "km³": "cubic kilometres",
                "kn": "knots",
                "l": "litres",
                "L": "Planck length",
                "L²": "Planck area",
                "L³": "Planck volume",
                "lb": "pounds",
                "m": "metres",
                "M": "Planck mass",
                "m²": "square metres",
                "Mb": "megabytes",
                "MiB": "megabytes",
                "Mbps": "megabytes per second",
                "Mcal": "megacalories",
                "mcg": "micrograms",
                "mCi": "millicuries",
                "MBq": "megabecquerels",
                "Mbit": "megabits",
                "Mibit": "megabits",
                "MJ": "megajoules",
                "MN": "meganewtons",
                "Mpa": "megapascals",
                "MW": "megawatts",
                "mkg": "metre-kilograms",
                "mg": "milligrams",
                "RT": "metric refrigeration tonnes",
                "mi": "miles",
                "MHz": "megahertz",
                "GHz": "gigahertz",
                "mi³": "cubic miles",
                "mpg": "miles per gallon",
                "mph": "miles per hour",
                "m.p.h": "miles per hour",
                "ml": "millilitres",
                "mm": "millimetres",
                "mN": "millinewtons",
                "min": "minutes",
                "mm²": "square millimetres",
                "mol": "moles",
                "N": "newtons",
                "nm": "nanometres",
                "Ncm": "newton centimetres",
                "Nm": "newton metres",
                "oz": "ounces",
                "ozf": "ounce force",
                "Pa": "pascals",
                "plm": "palms",
                "pc": "parsecs",
                "ppm": "parts per million",
                "ppb": "parts per billion",
                "Pb": "petabytes",
                "pc³": "cubic parsecs",
                "pdl": "poundals",
                "pt": "pints",
                "px": "pixels",
                "qt": "quarts",
                "rad": "radians",
                "rd": "rods",
                "s": "seconds",
                "Sv": "sieverts",
                "dm²": "square decimetres",
                "yd²": "square yards",
                "T": "Planck time",
                "Tbit": "terabits",
                "Tb": "terabytes",
                "tsp": "teaspoons",
                "tbsp": "tablespoons",
                "th": "thou",
                "VA": "volt-amperes",
                "V": "volts",
                "W": "watts",
                "yd": "yards",
                }



# -*- coding: utf-8 -*-

from __future__ import division, print_function, unicode_literals

import re
from collections import defaultdict
from io import open

import nltk

from normalise.detect import mod_path
from normalise.expand_EXPN import brown_common

with open('{}/data/wordlist.pickle'.format(mod_path), mode='rb') as file:
    wordlist = pickle.load(file)



# -*- coding: utf-8 -*-

tech_words = [
              'accelerometer',
              'android',
              'archive',
              'backup',
              'barcode',
              'bitmap',
              'backlink',
              'bandwidth',
              'booting',
              'bitcoin',
              'blog',
              'bluetooth',
              'bloatware',
              'benchmark',
              'binary',
              'broadband',
              'browser',
              'byte',
              'cache',
              'cellphone',
              'cybersecurity',
              'cyberspace',
              'compression',
              'cookie',
              'deamon',
              'debug',
              'ethernet',
              'encryption',
              'email',
              'emulator',
              'ethernet',
              'facebook',
              'fiberoptic',
              'framework',
              'freeware',
              'freeview',
              'firefox',
              'firewall',
              'google',
              'gmail',
              'hardware',
              'hotmail',
              'hypertext',
              'internet',
              'java',
              'joystick',
              'keyboard',
              'keypad',
              'laptop',
              'linux',
              'macintosh',
              'malware',
              'markup',
              'megabyte',
              'microsoft',
              'modem',
              'newbie',
              'overclock',
              'phising',
              'pixel',
              'python',
              'plugin',
              'rooting',
              'server',
              'software',
              'spam',
              'supercookie',
              'telecommunication',
              'trash',
              'underclock',
              'unicode',
              'unix',
              'virus',
              'website',
              'whitespace',
              'wireless',
              'wikipedia',
              'wi-fi',
              'windows'
              ]



# -*- coding: utf-8 -*-

from __future__ import division, print_function, unicode_literals

import sys
import re
import pickle
from io import open

from normalise.detect import mod_path
from normalise.tagger import tagify, NSWs, is_digbased, only_alpha

with open('{}/data/wordlist.pickle'.format(mod_path), mode='rb') as file:
    wordlist = pickle.load(file)

digits = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9']

if __name__ == "__main__":
    SPLT_dict = {ind: (nsw, tag) for ind, (nsw, tag)
                 in tagify(NSWs, verbose=False).items()
                 if tag == 'SPLT'}


def tag_SPLT(lst):
    """ Assign new index and tag 'SPLT-' to every part of a split token."""
    if len(lst) > 9:
        inc = round((1 / (len(lst))), 2)
    else:
        inc = 0.1
    ind = lst[0]
    out = {}
    for i in range(1, len(lst)):
        out.update({ind + inc * i: (lst[i], 'SPLT-')})
    return out


def split(dic, verbose=True):
    """ Form dictionary of SPLT tokens."""
    split_dict = {}
    done = 0
    for ind, (nsw, tag) in dic.items():
        if verbose:
            sys.stdout.write("\r{} of {} split".format(done, len(dic)))
            sys.stdout.flush()
        out = [ind]
        emph_list = []
        emph_match = emph_pattern.match(nsw)
        if emph_match:
            emph_list += emph_match.groups()
        else:
            emph_list += [nsw]
        hyph_list = []
        for nsw in emph_list:
            hyph_list.extend([item for item in nsw.split('-') if item])
        slash_list = []
        for nsw in hyph_list:
            slash_list.extend([item for item in nsw.split('/') if item])
        space_list = []
        for nsw in slash_list:
            space_list.extend([item for item in nsw.split(' ') if item])
        underscore_list = []
        for nsw in space_list:
            underscore_list.extend([item for item in nsw.split('_') if item])
        mixedalnum_list = []
        for nsw in underscore_list:
            mixedalnum_list.extend([item for item in mixedalnum_split(nsw)
                                    if item])
        updown_list = []
        for nsw in mixedalnum_list:
            updown_list.extend([item for item in split_updown(nsw) if item])
        mixedcase_list = []
        for nsw in updown_list:
            mixedcase_list.extend([item for item in mixedcase_split(nsw)
                                   if item])
        out.extend(mixedcase_list)
        split_dict.update(tag_SPLT(out))
        done += 1
    if verbose:
        sys.stdout.write("\r{} of {} split".format(done, len(dic)))
        sys.stdout.flush()
        print("\n")
    return split_dict


def retagify(dic, verbose=True):
    """ Retag each part of a SPLT token as 'SPLT-ALPHA', 'SPLT-NUMB' or
    'SPLT-MISC'.
    """
    out = {}
    for ind, (it, tag) in dic.items():
        if verbose:
            sys.stdout.write("\r{} of {} retagged".format(len(out), len(dic)))
            sys.stdout.flush()
        if len(it) > 100:
            out.update({ind: (it, tag + 'MISC')})
        if is_digbased(it):
            out.update({ind: (it, tag + 'NUMB')})
        elif (only_alpha(it) and
              (not mixedcase_pattern.match(it) or
               len(it) <= 3 or (it[-1] == 's' and not
               mixedcase_pattern.match(it[:-1])))):
                    out.update({ind: (it, tag + 'ALPHA')})
        else:
            out.update({ind: (it, tag + 'MISC')})
    if verbose:
        sys.stdout.write("\r{} of {} retagged".format(len(out), len(dic)))
        sys.stdout.flush()
        print("\n")
    return out


def split_updown(nsw):
    """ For tokens matching updown_pattern; split before or after penultimate
    upper-case character depending on whether resulting word is in wordlist.
    If neither group in wordlist, split before penultimate upper-case letter
    as default.
    """
    try:
        m = updown_pattern.match(nsw)
        if m:
            if (m.group(2) + m.group(3)).lower() in wordlist:
                return [m.group(1), m.group(2) + m.group(3)]
            elif m.group(3) in wordlist:
                return [m.group(1) + m.group(2), m.group(3)]
            else:
                return [m.group(1), m.group(2) + m.group(3)]
        else:
            return [nsw]
    except(KeyboardInterrupt, SystemExit):
        raise
    except:
        return nsw


def mixedalnum_split(nsw):
    """ Split tokens on transitions from letters to numbers or numbers to
    letters.
    """
    try:
        out = []
        ind = 0
        if nsw[0] in digits:
            cat = 'num'
        elif nsw[0].isalpha:
            cat = 'let'
        else:
            cat = 'punc'
        for i in range(1, len(nsw)):
            if nsw[i] in digits:
                if cat == 'num' or cat == 'punc':
                    pass
                else:
                    out.append(nsw[ind:i])
                    cat = 'num'
                    ind = i
            elif nsw[i].isalpha():
                if cat == 'let' or cat == 'punc':
                    pass
                else:
                    out.append(nsw[ind:i])
                    cat = 'let'
                    ind = i
            else:
                pass
        out.append(nsw[ind:])
        return out
    except(KeyboardInterrupt, SystemExit):
        raise
    except:
        return nsw


def mixedcase_split(nsw):
    """ Split tokens on transitions from upper- to lower- or lower- to
    upper-case.
    """
    try:
        if nsw.isalpha():
            if nsw.istitle():
                return [nsw]
            else:
                out = []
                ind = 0
                if nsw[0].isupper():
                    cat = 'up'
                else:
                    cat = 'low'
                for i in range(1, len(nsw)):
                    if nsw[i].isupper():
                        if cat == 'up':
                            pass
                        else:
                            out.append(nsw[ind:i])
                            cat = 'up'
                            ind = i
                    else:
                        if cat == 'low':
                            pass
                        elif nsw[i - 1].isupper():
                            cat = 'low'
                            pass
                        else:
                            out.append(nsw[ind:i])
                            cat = 'low'
                            ind = i
                out.append(nsw[ind:])
                return out
        else:
            return [nsw]
    except(KeyboardInterrupt, SystemExit):
        raise
    except:
        return nsw


hyphen_pattern = re.compile('''
(([\'\.]?                       # optional "'" or "."
[A-Za-z]                        # letter
[\'\.]?)                        # optional "'" or "."
+                               # lines 1-3 repeated one or more times
(-|/|\s|(-&-))                  # followed by '-', '/', or '-&-'
)+                              # all of the above repeated 1+ times
([\'\.]?[A-Za-z][\.\']?)*       # optional final 'word' (same as lines 1-3)
$                               # end of string
''', re.VERBOSE)

mixedcase_pattern = re.compile('''
([A-Z]{2,}[a-z]) |    # 2 or more capitals, a lowercase OR
(.*[a-z][A-Z])        # any number of chars, a lowercase, an uppercase
''', re.VERBOSE)

emph_pattern = re.compile('''
([\*~<:]+)                      # '*' or '~' repeated one or more times
((?:[A-Za-z]+[-/\']?)+[A-Za-z]+)# optionally hyphenated 'word'
([\*~>:]+)                      # '*' or '~' repeated one or more times
$                               # end of string
''', re.VERBOSE)

updown_pattern = re.compile('''
([A-Z]+?)                       # 1 or more capitals (non-greedy) (capturing)
([A-Z])                         # a capital (capturing)
([a-z]+)                        # 1 or more lowercase (capturing)
$
''', re.VERBOSE)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()
setup(
    name='normalise',
    platforms=['any'],
    packages=find_packages(exclude=['tests', 'evaluation']),
    include_package_data=True,
    version='0.1.6',
    description='A module to normalise non-standard words in text',
    long_description=long_description,
    author='Elliot Ford, Emma Flint',
    author_email='ef355@cam.ac.uk',
    license="GPL",
    url='https://github.com/EFord36/normalise',
    download_url='https://github.com/EFord36/normalise/tarball/0.1.6',
    keywords=['normalisation', 'text', 'pre-processing'],
    classifiers=[
        "Natural Language :: English",
        "Topic :: Text Processing",
        ("License :: OSI Approved :: "
        "GNU General Public License v3 or later (GPLv3+)")
    ],
    install_requires=['nltk', 'scikit-learn', 'numpy', 'roman', 'scipy'],
    entry_points={'console_scripts': ['normalise=normalise.command_line:main']}
)



# -*- coding: utf-8 -*-

from __future__ import division, print_function, unicode_literals

import re
import pickle
from io import open

from math import log
from nltk import FreqDist as fd
from normalise.detect import wordlist, mod_path
from normalise.expand_NUMB import expand_NUM
from normalise.tagger import has_digit
with open('{}/data/word_tokenized_lowered.pickle'.format(mod_path), mode='rb') as f:
    word_tokenized_lowered = pickle.load(f)


def expand_HTAG(word):
    """Expand tokens tagged HTAG."""
    try:
        m = hashtag_pattern.match(word)
        string = m.group(2)
        exp = 'hashtag'
        if string in wordlist:
            exp = exp + " " + string
        else:
            exp = exp + " " + infer_spaces(m.group(2))
        if m.group(3):
            exp = exp + " " + expand_NUM(m.group(3))
        return exp
    except(KeyboardInterrupt, SystemExit):
        raise
    except:
        return word


def email_word(word):
    if not word:
        return ''
    elif word.isdigit():
        return ' '.join(word)
    elif word in wordlist:
        return word
    elif word in ['co', 'com', 'org']:
        return word
    else:
        if email_dig_pattern.match(word):
            m = email_dig_pattern.match(word)
            groups = [m.group(1), m.group(2), m.group(3)]
            return ' '.join([email_word(group) for group in groups if group])
        elif len(word) < 4:
            return ' '.join(word.upper())
        else:
            return expand_HTAG(word)


def expand_URL(word):
    """Expand tokens tagged URL."""
    try:
        if '@' in word:
            out = ''
            current = ''
            for c in word:
                if c == '.':
                    # out += ' ' + expand_HTAG(current) + ' dot'
                    out += ' {} dot'.format(current if current in wordlist else email_word(current))
                    current = ''
                elif c == '@':
                    # out += ' ' + expand_HTAG(current) + ' at'
                    out += ' {} at'.format(current if current in wordlist else email_word(current))
                    current = ''
                else:
                    current += c
            if current:
                out += ' ' + current
            return out
        starts = {"http://": "", "https://": "", "www.": "W W W dot"}
        ends = {".com": "dot com", ".org": "dot org",
                ".org.uk": "dot org dot U K", ".co.uk": "dot co dot U K"}
        m = urlstart_pattern.match(word)
        n = urlend_pattern.match(word)
        exp = ''
        if m.group(1) and n:
            start = m.group(1)
            middle = urlend_pattern.match(m.group(2))
            end = middle.group(2)
            exp += (starts[start] + " "
                    + infer_spaces(middle.group(1))
                    + " " + ends[end])
        elif n:
            middle = n.group(1)
            end = n.group(2)
            exp += infer_spaces(middle) + " " + ends[end]
        else:
            return word
        return exp
    except(KeyboardInterrupt, SystemExit):
        raise
    except:
        return word


# Build a cost dict, assuming Zipf's law and cost = -math.log(probability).
brown = word_tokenized_lowered[:1161192]
words = [w for w, freq in fd(brown).most_common()]
wordcost = dict((k, log((i + 1) * log(len(words))))
                for i, k in enumerate(words))
maxword = max(len(x) for x in words)


def infer_spaces(s):
    """Uses dynamic programming to infer the location of spaces in a string
    without spaces."""

    # Find the best match for the i first characters, assuming cost has
    # been built for the i-1 first characters.
    # Returns a pair (match_cost, match_length).
    def best_match(i):
        candidates = enumerate(reversed(cost[max(0, i - maxword):i]))
        return min((c + wordcost.get(s[i - k - 1:i], 9e999), k + 1)
                   for k, c in candidates)

    # Build the cost array.
    cost = [0]
    for i in range(1, len(s) + 1):
        c, k = best_match(i)
        cost.append(c)

    # Backtrack to recover the minimal-cost string.
    out = []
    i = len(s)
    while i > 0:
        c, k = best_match(i)
        assert c == cost[i]
        out.append(s[i - k: i])
        i -= k

    return " ".join(reversed(out))

hashtag_pattern = re.compile('''
(\#)
([A-Za-z]+
[_-]?
[A-Za-z]*)
([0-9]*)
''', re.VERBOSE)

urlstart_pattern = re.compile('''
(https?://|       # 'http' followed by optional 's', then '://' OR
www\.)            # 'www.'
(.*)              # followed by anything
''', re.VERBOSE | re.IGNORECASE)

urlend_pattern = re.compile('''
(.*)              # any number of characters
(\.com|           # '.com' OR
\.org|            # '.org' OR
\.org\.uk|        # '.org.uk' OR
\.co\.uk)         # '.co.uk'
$                 # end of string
''', re.VERBOSE | re.IGNORECASE)

email_dig_pattern = re.compile('''
([A-Za-z]*)       # 0 or more letters
([0-9]+)          # 1 or more digits
([A-Za-z]*)       # 0 or more letters
$                 # end of string
''', re.VERBOSE | re.IGNORECASE)



# -*- coding: utf-8 -*-

from __future__ import division, print_function, unicode_literals

import sys
import re
import pickle
from io import open

from normalise.detect import mod_path
from normalise.class_ALPHA import triple_rep
from normalise.spellcheck import correct
from normalise.expand_EXPN import expand_EXPN
from normalise.expand_HTAG import expand_HTAG, expand_URL
from normalise.expand_NUMB import (expand_NUM, expand_NDIG, expand_NORD,
                                   expand_NYER, expand_PRCT, expand_MONEY,
                                   expand_NTIME, expand_NRANGE, expand_NTEL,
                                   expand_NDATE, expand_NSCI)

with open('{}/data/wordlist.pickle'.format(mod_path), mode='rb') as file:
    wordlist = pickle.load(file)

func_dict = {
             'EXPN': 'expand_EXPN(nsw, ind, text, user_abbrevs=user_abbrevs)',
             'LSEQ': 'expand_LSEQ(nsw)',
             'WDLK': 'expand_WDLK(nsw)',
             'NUM': 'expand_NUM(nsw)',
             'NORD': 'expand_NORD((ind, (nsw, tag, ntag)), text)',
             'NRANGE': 'expand_NRANGE(nsw)',
             'NDIG': 'expand_NDIG(nsw)',
             'NTIME': 'expand_NTIME(nsw)',
             'NDATE': 'expand_NDATE(nsw, variety=variety)',
             'NADDR': 'expand_NYER(nsw)',
             'NTEL': 'expand_NTEL(nsw)',
             'NSCI': 'expand_NSCI(nsw)',
             'NYER': 'expand_NYER(nsw)',
             'MONEY': 'expand_MONEY((ind, (nsw, tag, ntag)), text)',
             'PRCT': 'expand_PRCT(nsw)',
             'PROF': 'expand_PROF(nsw)',
             'URL': 'expand_URL(nsw)',
             'HTAG': 'expand_HTAG(nsw)',
             'NONE': 'expand_NONE(nsw)'
             }


def expand_all(dic, text, verbose=True, variety='BrE', user_abbrevs={}):
    out = {}
    for ind, (nsw, tag, ntag) in dic.items():
        if verbose:
            sys.stdout.write("\r{} of {} expanded".format(len(out), len(dic)))
            sys.stdout.flush()
        out.update({ind: (nsw, tag, ntag, (eval(func_dict[ntag])))})
    if verbose:
        sys.stdout.write("\r{} of {} expanded".format(len(out), len(dic)))
        sys.stdout.flush()
        print("\n")
    return out


def expand_NONE(nsw):
    """For nsws tagged 'NONE', return 'and' if nsw is '&', otherwise return
       nothing."""
    if nsw == '&':
        return 'and'
    else:
        return ''


def expand_PROF(w):
    try:
        """Return 'original' rude word from asterisked 'WDLK'."""
        rude = ['ass', 'arse', 'asshole', 'balls', 'bitch', 'cunt',
                'cock', 'crap', 'cum', 'damn' 'dick', 'fuck', 'motherfucker',
                'pussy','shit', 'tits', 'twat', 'wank', 'wanker']
        candidates = [r for r in rude if len(r) == len(w)]
        final = ''
        ind = 0
        if not candidates:
            return w
        else:
            while not final and ind < len(candidates):
                r = candidates[ind]
                match = True
                for i in range(len(r)):
                    if r[i] != w[i] and w[i] != '*':
                        match = False
                if match:
                    final += r
                ind += 1
            if final:
                return final
            else:
                return w
    except (KeyboardInterrupt, SystemExit):
        raise
    except:
        return w


def expand_WDLK(word):
    """Expand 'WDLK' tokens."""
    try:
        if word.lower() in wordlist:
            return word
        elif word[0].isupper() and word[1:].islower():
            return word
        elif triple_rep(word):
            return expand_FNSP(word)
        else:
            return correct(word)
    except (KeyboardInterrupt, SystemExit):
        raise
    except:
        return word


def expand_FNSP(w):
    """Return 'original' word from FNSP."""
    try:
        reg = create_regexp(w)
        final = ''
        for e in wordlist:
            m = re.match(reg, e)
            if m:
                final = m.string
                break
        if final:
            return final
        else:
            red_word = w[0]
            for i in range(1, len(w)):
                if w[i] != w[i - 1]:
                    red_word += w[i]
            return red_word
    except (KeyboardInterrupt, SystemExit):
        raise
    except:
        return w


def create_regexp(w):
    """Return regular expression representing word with repeated cs 1+ times.
    """
    regexp = w[0]
    for i in range(1, len(w)):
        if w[i] == w[i - 1]:
            if regexp[-1] != '+':
                regexp += '+'
        else:
            regexp += w[i]
    regexp += '$'
    return regexp


def expand_LSEQ(word):
    """Expand 'LSEQ' tokens to a series of letters."""
    try:
        out = ''
        if word[0].isalpha():
            out += word[0].upper()
        for c in word[1:]:
            if c.isalpha():
                out += ' '
                out += c.upper()
        return out
    except (KeyboardInterrupt, SystemExit):
        raise
    except:
        return word
